/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.api.IClassTest;
import com.tocea.easycoverage.framework.utils.WeakImplementationException;



/**
 * This type describes a simple invocation test.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class NPECloneTest implements IClassTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(NPECloneTest.class);
    private final Object                  testedObject;
    
    
    
    /**
     * Builds a simple invocation test .
     * 
     * @param _testedObject
     *            the tested class.
     */
    public NPECloneTest(final Object _testedObject) {
    
    
        super();
        this.testedObject = _testedObject;
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.IClassTest#performTest()
     */
    public void performTest() throws WeakImplementationException {
    
    
        LOGGER.debug("Processing the method clone to win against NPE faults.");
        
        if (this.testedObject instanceof Cloneable) {
            Object clone;
            try {
                final Method method = this.testedObject.getClass().getMethod("clone");
                method.setAccessible(true);
                clone = method.invoke(this.testedObject);
                method.setAccessible(false);
                if (clone == null) {
                    throw new WeakImplementationException("The method clone() should never returns null.", this.getClass()
                            .getCanonicalName());
                }
            } catch (final IllegalArgumentException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final SecurityException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final IllegalAccessException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final InvocationTargetException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final NoSuchMethodException e) {
                LOGGER.error(e.getMessage(), e);
                
            }
        }
    }
    
}
